module demo(in_clk, in_reset, in_enable, in_key, in_busy, out_data, out_start, out_data_dac1, out_data_dac2, out_led);

input in_clk;
input in_reset;
input in_enable;
input in_busy;
input in_key;


output out_start;
output out_led;

output [15:0] out_data;
output [11:0] out_data_dac1;
output [11:0] out_data_dac2;


reg start;

reg [15:0] data;
reg [11:0] pattern_1;
reg [11:0] pattern_2;

reg [3:0] counter_pattern;

reg key1;
reg key2;


reg busy1;
reg busy2;


reg [3:0] mode;
reg [3:0] counter_pac;
reg [15:0] timer_delay;

reg [7:0] counter;

reg led;

wire key;
wire busy;


assign out_start = start;
assign out_data = data;
assign out_data_dac1 = pattern_1;
assign out_data_dac2 = pattern_2;


assign key = key1 & key2;
assign busy = !busy1 & busy2;

assign out_led = led;

//key1
always @(posedge in_clk)
	begin
		if(in_reset == 1'd0)
			begin
				key1 <= 1'd1;
			end
		else
			begin
				if(in_enable == 1'd1) key1 <= in_key;
				else key1 <= 1'd1;
 			end
	end

	
//key2
always @(posedge in_clk)
	begin
		if(in_reset == 1'd0)
			begin
				key2 <= 1'd0;
			end
		else
			begin
				if(in_enable == 1'd1) key2 <= ~key1;
				else key2 <= 1'd0;
			end
	end
	
	
//busy1
always @(posedge in_clk)
	begin
		if(in_reset == 1'd0)
			begin
				busy1 <= 1'd0;
			end
		else
			begin
				if(in_enable == 1'd1) busy1 <= in_busy;
				else busy1 <= 1'd0;
			end
	end

//busy2
always @(posedge in_clk)
	begin
		if(in_reset == 1'd0)
			begin
				busy2 <= 1'd0;
			end
		else
			begin
				if(in_enable == 1'd1) busy2 <= busy1;
				else busy2 <= 1'd0;
			end
	end

//mode
always @(posedge in_clk)
	begin
		if(in_reset == 1'd0)
			begin
				mode <= 4'd0;
			end
		else
			begin
				if(in_enable == 1'd1)
					begin
						if(key == 1'd1 && mode < 4'd4) mode <= mode + 1;
						else if(mode == 4'd4) mode <= 4'd0;
					end
				else mode <= 4'd0;
			end
	end




//data
always @(posedge in_clk)
	begin
		if(in_reset == 1'd0)
			begin
				data <= 16'd0;
			end
		else
			begin
				if(in_enable == 1'd1)
					begin
						if(mode == 4'd0)
							begin
								if(counter_pac == 4'd0) data <= 16'h0104;
								else if(counter_pac == 4'd1) data <= 16'h0280;
								else if(counter_pac == 4'd2) data <= 16'h0303;
								else if(counter_pac == 4'd3) data <= 16'h0480;
							end
						else if(mode == 4'd1)
							begin
								if(counter_pac == 4'd0) data <= 16'h0480;
								else if(counter_pac == 4'd1) data <= 16'h0280;
								else if(counter_pac == 4'd2) data <= 16'h0302;
								else if(counter_pac == 4'd3) data <= 16'h0144;								
							end
						else if(mode == 4'd2)
							begin
								if(counter_pac == 4'd0) data <= 16'h0480;
								else if(counter_pac == 4'd1) data <= 16'h0280;
								else if(counter_pac == 4'd2) data <= 16'h0301;
								else if(counter_pac == 4'd3) data <= 16'h0184;								
							end
						else if(mode == 4'd3)
							begin
								if(counter_pac == 4'd0) data <= 16'h0400;
								else if(counter_pac == 4'd1) data <= 16'h0280;
								else if(counter_pac == 4'd2) data <= 16'h0300;
								else if(counter_pac == 4'd3) data <= 16'h01C4;								
							end							
					end
				else data <= 16'd0;
			end
	end
	
	
	
//counter_pac
always @(posedge in_clk)
		begin
			if(in_reset == 1'd0)
				begin
					counter_pac <= 4'd0;
				end
			else
				begin
					if(in_enable == 1'd1)
						begin
							if(key == 1'd1) counter_pac <= 4'd0;
							else if(busy == 1'd1 && counter_pac < 4'd4) counter_pac <= counter_pac + 1;
						end
					else counter_pac <= 4'd0;
				end
		end


		
//timer_delay
always @(posedge in_clk)
	begin
		if(in_reset == 1'd0)
			begin
				timer_delay <= 16'd0;
			end
		else
			begin
				if(start == 1'd0 && timer_delay < 16'd3500) timer_delay <= timer_delay + 1;
				else if(start == 1'd1) timer_delay <= 16'd0;
			end
	end
	

	
//start
always @(posedge in_clk)
	begin
		if(in_reset == 1'd0)
			begin
				start <= 1'd0;
			end
		else
			begin
				if(in_enable == 1'd1)
					begin
						if(timer_delay == 16'd3500 && counter_pac != 4'd4 && counter_pac != 4'd4) start <= 1'd1;
						else if(timer_delay == 16'd0 && in_busy == 1'd0)  start <= 1'd0;
					end
				else start <= 1'd0;
			end
	end
	

	


/*counter
always @(posedge in_clk)
	begin
		if(in_reset == 1'd0)
			begin
				counter <= 8'd0;
			end
		else
			begin
				if(in_enable == 1'd1)
					begin
						if(counter != 8'd50) counter <= counter + 1;
						else counter <= 8'd0;
					end
				else counter <= 8'd0;
			end
	end
*/

//counter_pattern

always @(posedge in_clk)
	begin
		if(in_reset == 1'h0) counter_pattern <= 4'h0;
		else
			begin
				if(in_enable == 1'h1)
					begin
						if(counter_pattern < 4'd4) counter_pattern <= counter_pattern + 1;
						else if(counter_pattern == 4'd4) counter_pattern <= 4'h0;
					end
				else counter_pattern <= 4'h0;
			end
	end


//counter
always @(posedge in_clk)
	begin
		if(in_reset == 1'h0) counter <= 8'h00;
		else
			begin
				if(in_enable == 1'h1)
					begin
						if(counter_pattern == 4'h3 && counter < 8'd100) counter <= counter + 1;
						else if(counter == 8'd100) counter <= 8'd0;
					end
				else counter <= 8'h00;
			end
	end
		
	
	
//pattern_1
always @(posedge in_clk)
	begin
		if(in_reset == 1'h0) pattern_1 <= 12'h000;
		else
			begin
				if(in_enable == 1'h1)
					begin
						if(counter == 8'd0) pattern_1 <= 12'd2047; 
						else if(counter == 8'd1) pattern_1 <= 12'd2176; 
						else if(counter == 8'd2) pattern_1 <= 12'd2304; 
						else if(counter == 8'd3) pattern_1 <= 12'd2431; 
						else if(counter == 8'd4) pattern_1 <= 12'd2556; 
						else if(counter == 8'd5) pattern_1 <= 12'd2680; 
						else if(counter == 8'd6) pattern_1 <= 12'd2801; 
						else if(counter == 8'd7) pattern_1 <= 12'd2919; 
						else if(counter == 8'd8) pattern_1 <= 12'd3033; 
						else if(counter == 8'd9) pattern_1 <= 12'd3144; 
						else if(counter == 8'd10) pattern_1 <= 12'd3250; 
						else if(counter == 8'd11) pattern_1 <= 12'd3352; 
						else if(counter == 8'd12) pattern_1 <= 12'd3449; 
						else if(counter == 8'd13) pattern_1 <= 12'd3540; 
						else if(counter == 8'd14) pattern_1 <= 12'd3625; 
						else if(counter == 8'd15) pattern_1 <= 12'd3703; 
						else if(counter == 8'd16) pattern_1 <= 12'd3776; 
						else if(counter == 8'd17) pattern_1 <= 12'd3841; 
						else if(counter == 8'd18) pattern_1 <= 12'd3900; 
						else if(counter == 8'd19) pattern_1 <= 12'd3951; 
						else if(counter == 8'd20) pattern_1 <= 12'd3994; 
						else if(counter == 8'd21) pattern_1 <= 12'd4030; 
						else if(counter == 8'd22) pattern_1 <= 12'd4058; 
						else if(counter == 8'd23) pattern_1 <= 12'd4078; 
						else if(counter == 8'd24) pattern_1 <= 12'd4090; 
						else if(counter == 8'd25) pattern_1 <= 12'd4095; 
						else if(counter == 8'd26) pattern_1 <= 12'd4090; 
						else if(counter == 8'd27) pattern_1 <= 12'd4078; 
						else if(counter == 8'd28) pattern_1 <= 12'd4058; 
						else if(counter == 8'd29) pattern_1 <= 12'd4030; 
						else if(counter == 8'd30) pattern_1 <= 12'd3994; 
						else if(counter == 8'd31) pattern_1 <= 12'd3951; 
						else if(counter == 8'd32) pattern_1 <= 12'd3900; 
						else if(counter == 8'd33) pattern_1 <= 12'd3841; 
						else if(counter == 8'd34) pattern_1 <= 12'd3776; 
						else if(counter == 8'd35) pattern_1 <= 12'd3703; 
						else if(counter == 8'd36) pattern_1 <= 12'd3625; 
						else if(counter == 8'd37) pattern_1 <= 12'd3540; 
						else if(counter == 8'd38) pattern_1 <= 12'd3449; 
						else if(counter == 8'd39) pattern_1 <= 12'd3352; 
						else if(counter == 8'd40) pattern_1 <= 12'd3250; 
						else if(counter == 8'd41) pattern_1 <= 12'd3144; 
						else if(counter == 8'd42) pattern_1 <= 12'd3033; 
						else if(counter == 8'd43) pattern_1 <= 12'd2919; 
						else if(counter == 8'd44) pattern_1 <= 12'd2801; 
						else if(counter == 8'd45) pattern_1 <= 12'd2680; 
						else if(counter == 8'd46) pattern_1 <= 12'd2556; 
						else if(counter == 8'd47) pattern_1 <= 12'd2431; 
						else if(counter == 8'd48) pattern_1 <= 12'd2304; 
						else if(counter == 8'd49) pattern_1 <= 12'd2176; 
						else if(counter == 8'd50) pattern_1 <= 12'd2047; 
						else if(counter == 8'd51) pattern_1 <= 12'd1918; 
						else if(counter == 8'd52) pattern_1 <= 12'd1790; 
						else if(counter == 8'd53) pattern_1 <= 12'd1663; 
						else if(counter == 8'd54) pattern_1 <= 12'd1538; 
						else if(counter == 8'd55) pattern_1 <= 12'd1414; 
						else if(counter == 8'd56) pattern_1 <= 12'd1293; 
						else if(counter == 8'd57) pattern_1 <= 12'd1175; 
						else if(counter == 8'd58) pattern_1 <= 12'd1061; 
						else if(counter == 8'd59) pattern_1 <= 12'd950; 
						else if(counter == 8'd60) pattern_1 <= 12'd844; 
						else if(counter == 8'd61) pattern_1 <= 12'd742; 
						else if(counter == 8'd62) pattern_1 <= 12'd645; 
						else if(counter == 8'd63) pattern_1 <= 12'd554; 
						else if(counter == 8'd64) pattern_1 <= 12'd469; 
						else if(counter == 8'd65) pattern_1 <= 12'd391; 
						else if(counter == 8'd66) pattern_1 <= 12'd318; 
						else if(counter == 8'd67) pattern_1 <= 12'd253; 
						else if(counter == 8'd68) pattern_1 <= 12'd194; 
						else if(counter == 8'd69) pattern_1 <= 12'd143; 
						else if(counter == 8'd70) pattern_1 <= 12'd100; 
						else if(counter == 8'd71) pattern_1 <= 12'd64; 
						else if(counter == 8'd72) pattern_1 <= 12'd36; 
						else if(counter == 8'd73) pattern_1 <= 12'd16; 
						else if(counter == 8'd74) pattern_1 <= 12'd4; 
						else if(counter == 8'd75) pattern_1 <= 12'd0; 
						else if(counter == 8'd76) pattern_1 <= 12'd4; 
						else if(counter == 8'd77) pattern_1 <= 12'd16; 
						else if(counter == 8'd78) pattern_1 <= 12'd36; 
						else if(counter == 8'd79) pattern_1 <= 12'd64; 
						else if(counter == 8'd80) pattern_1 <= 12'd100; 
						else if(counter == 8'd81) pattern_1 <= 12'd143; 
						else if(counter == 8'd82) pattern_1 <= 12'd194; 
						else if(counter == 8'd83) pattern_1 <= 12'd253; 
						else if(counter == 8'd84) pattern_1 <= 12'd318; 
						else if(counter == 8'd85) pattern_1 <= 12'd391; 
						else if(counter == 8'd86) pattern_1 <= 12'd469; 
						else if(counter == 8'd87) pattern_1 <= 12'd554; 
						else if(counter == 8'd88) pattern_1 <= 12'd645; 
						else if(counter == 8'd89) pattern_1 <= 12'd742; 
						else if(counter == 8'd90) pattern_1 <= 12'd844; 
						else if(counter == 8'd91) pattern_1 <= 12'd950; 
						else if(counter == 8'd92) pattern_1 <= 12'd1061; 
						else if(counter == 8'd93) pattern_1 <= 12'd1175; 
						else if(counter == 8'd94) pattern_1 <= 12'd1293; 
						else if(counter == 8'd95) pattern_1 <= 12'd1414; 
						else if(counter == 8'd96) pattern_1 <= 12'd1538; 
						else if(counter == 8'd97) pattern_1 <= 12'd1663; 
						else if(counter == 8'd98) pattern_1 <= 12'd1790; 
						else if(counter == 8'd99) pattern_1 <= 12'd1918; 

 

					end
				else pattern_1 <= 12'h000;
			end
	end
	
	
//pattern_2
always @(posedge in_clk)
	begin
		if(in_reset == 1'd0)
			begin
				pattern_2 <= 12'd0;
			end
		else
			begin
				if(in_enable == 1'd1)
					begin
						if(counter == 8'd0) pattern_2 <= 12'd0; 
						else if(counter == 8'd1) pattern_2 <= 12'd40; 
						else if(counter == 8'd2) pattern_2 <= 12'd80; 
						else if(counter == 8'd3) pattern_2 <= 12'd120; 
						else if(counter == 8'd4) pattern_2 <= 12'd160; 
						else if(counter == 8'd5) pattern_2 <= 12'd200; 
						else if(counter == 8'd6) pattern_2 <= 12'd240; 
						else if(counter == 8'd7) pattern_2 <= 12'd280; 
						else if(counter == 8'd8) pattern_2 <= 12'd320; 
						else if(counter == 8'd9) pattern_2 <= 12'd360; 
						else if(counter == 8'd10) pattern_2 <= 12'd400; 
						else if(counter == 8'd11) pattern_2 <= 12'd440; 
						else if(counter == 8'd12) pattern_2 <= 12'd480; 
						else if(counter == 8'd13) pattern_2 <= 12'd520; 
						else if(counter == 8'd14) pattern_2 <= 12'd560; 
						else if(counter == 8'd15) pattern_2 <= 12'd600; 
						else if(counter == 8'd16) pattern_2 <= 12'd640; 
						else if(counter == 8'd17) pattern_2 <= 12'd680; 
						else if(counter == 8'd18) pattern_2 <= 12'd720; 
						else if(counter == 8'd19) pattern_2 <= 12'd760; 
						else if(counter == 8'd20) pattern_2 <= 12'd800; 
						else if(counter == 8'd21) pattern_2 <= 12'd840; 
						else if(counter == 8'd22) pattern_2 <= 12'd880; 
						else if(counter == 8'd23) pattern_2 <= 12'd920; 
						else if(counter == 8'd24) pattern_2 <= 12'd960; 
						else if(counter == 8'd25) pattern_2 <= 12'd1000; 
						else if(counter == 8'd26) pattern_2 <= 12'd1040; 
						else if(counter == 8'd27) pattern_2 <= 12'd1080; 
						else if(counter == 8'd28) pattern_2 <= 12'd1120; 
						else if(counter == 8'd29) pattern_2 <= 12'd1160; 
						else if(counter == 8'd30) pattern_2 <= 12'd1200; 
						else if(counter == 8'd31) pattern_2 <= 12'd1240; 
						else if(counter == 8'd32) pattern_2 <= 12'd1280; 
						else if(counter == 8'd33) pattern_2 <= 12'd1320; 
						else if(counter == 8'd34) pattern_2 <= 12'd1360; 
						else if(counter == 8'd35) pattern_2 <= 12'd1400; 
						else if(counter == 8'd36) pattern_2 <= 12'd1440; 
						else if(counter == 8'd37) pattern_2 <= 12'd1480; 
						else if(counter == 8'd38) pattern_2 <= 12'd1520; 
						else if(counter == 8'd39) pattern_2 <= 12'd1560; 
						else if(counter == 8'd40) pattern_2 <= 12'd1600; 
						else if(counter == 8'd41) pattern_2 <= 12'd1640; 
						else if(counter == 8'd42) pattern_2 <= 12'd1680; 
						else if(counter == 8'd43) pattern_2 <= 12'd1720; 
						else if(counter == 8'd44) pattern_2 <= 12'd1760; 
						else if(counter == 8'd45) pattern_2 <= 12'd1800; 
						else if(counter == 8'd46) pattern_2 <= 12'd1840; 
						else if(counter == 8'd47) pattern_2 <= 12'd1880; 
						else if(counter == 8'd48) pattern_2 <= 12'd1920; 
						else if(counter == 8'd49) pattern_2 <= 12'd1960; 
						else if(counter == 8'd50) pattern_2 <= 12'd2000; 
						else if(counter == 8'd51) pattern_2 <= 12'd2040; 
						else if(counter == 8'd52) pattern_2 <= 12'd2080; 
						else if(counter == 8'd53) pattern_2 <= 12'd2120; 
						else if(counter == 8'd54) pattern_2 <= 12'd2160; 
						else if(counter == 8'd55) pattern_2 <= 12'd2200; 
						else if(counter == 8'd56) pattern_2 <= 12'd2240; 
						else if(counter == 8'd57) pattern_2 <= 12'd2280; 
						else if(counter == 8'd58) pattern_2 <= 12'd2320; 
						else if(counter == 8'd59) pattern_2 <= 12'd2360; 
						else if(counter == 8'd60) pattern_2 <= 12'd2400; 
						else if(counter == 8'd61) pattern_2 <= 12'd2440; 
						else if(counter == 8'd62) pattern_2 <= 12'd2480; 
						else if(counter == 8'd63) pattern_2 <= 12'd2520; 
						else if(counter == 8'd64) pattern_2 <= 12'd2560; 
						else if(counter == 8'd65) pattern_2 <= 12'd2600; 
						else if(counter == 8'd66) pattern_2 <= 12'd2640; 
						else if(counter == 8'd67) pattern_2 <= 12'd2680; 
						else if(counter == 8'd68) pattern_2 <= 12'd2720; 
						else if(counter == 8'd69) pattern_2 <= 12'd2760; 
						else if(counter == 8'd70) pattern_2 <= 12'd2800; 
						else if(counter == 8'd71) pattern_2 <= 12'd2840; 
						else if(counter == 8'd72) pattern_2 <= 12'd2880; 
						else if(counter == 8'd73) pattern_2 <= 12'd2920; 
						else if(counter == 8'd74) pattern_2 <= 12'd2960; 
						else if(counter == 8'd75) pattern_2 <= 12'd3000; 
						else if(counter == 8'd76) pattern_2 <= 12'd3040; 
						else if(counter == 8'd77) pattern_2 <= 12'd3080; 
						else if(counter == 8'd78) pattern_2 <= 12'd3120; 
						else if(counter == 8'd79) pattern_2 <= 12'd3160; 
						else if(counter == 8'd80) pattern_2 <= 12'd3200; 
						else if(counter == 8'd81) pattern_2 <= 12'd3240; 
						else if(counter == 8'd82) pattern_2 <= 12'd3280; 
						else if(counter == 8'd83) pattern_2 <= 12'd3320; 
						else if(counter == 8'd84) pattern_2 <= 12'd3360; 
						else if(counter == 8'd85) pattern_2 <= 12'd3400; 
						else if(counter == 8'd86) pattern_2 <= 12'd3440; 
						else if(counter == 8'd87) pattern_2 <= 12'd3480; 
						else if(counter == 8'd88) pattern_2 <= 12'd3520; 
						else if(counter == 8'd89) pattern_2 <= 12'd3560; 
						else if(counter == 8'd90) pattern_2 <= 12'd3600; 
						else if(counter == 8'd91) pattern_2 <= 12'd3640; 
						else if(counter == 8'd92) pattern_2 <= 12'd3680; 
						else if(counter == 8'd93) pattern_2 <= 12'd3720; 
						else if(counter == 8'd94) pattern_2 <= 12'd3760; 
						else if(counter == 8'd95) pattern_2 <= 12'd3800; 
						else if(counter == 8'd96) pattern_2 <= 12'd3840; 
						else if(counter == 8'd97) pattern_2 <= 12'd3880; 
						else if(counter == 8'd98) pattern_2 <= 12'd3920; 
						else if(counter == 8'd99) pattern_2 <= 12'd3960; 
				
					end
				else pattern_2 <= 12'd0;					
			end
	end


	
	
//led
always @(posedge in_clk)
	begin
		if(in_reset == 1'd0)
			begin
				led <= 1'd0;
			end
		else
			begin
				if(in_enable == 1'd1 ) led <= 1'd1;
				else led <= 1'd0;
			end
			
	end

endmodule