module spi_master_dac(in_clk, in_reset, in_sdo, in_data_out, in_start, out_sclk, out_ss, out_sdio, out_busy);

input in_clk;
input in_reset;
input in_sdo;
input in_start;

input [15:0] in_data_out;


output out_sclk;
output out_ss;
output out_sdio;
output out_busy;






reg sclk;
reg sclk_buf;
reg sdio;
reg ss;
reg busy;
reg new_data;


reg [15:0] dout;
reg [7:0] counter_data;
reg [3:0] timer_sclk;
reg [3:0] timer_delay;

wire sclk_pos;
wire sclk_neg;

assign sclk_neg = !sclk & sclk_buf;
assign sclk_pos = !sclk_buf & sclk;

assign out_sdio = sdio;
assign out_sclk = sclk;
assign out_ss = ss;
assign out_busy = busy;





//timer_sclk
always @(posedge in_clk)
	begin
		if(in_reset == 1'd0) 
			begin
				timer_sclk <= 4'd0;
			end
		else
			begin
				if(in_start == 1'd1 && timer_sclk < 4'd5) timer_sclk <= timer_sclk + 1;
				else timer_sclk <= 4'd0;
			end
	end



//sclk
always @(posedge in_clk)
	begin
		if(in_reset == 1'd0)
			begin
				sclk <= 1'h1;
			end
		else 
			begin
				if(in_start == 1'd1)
					begin
						if(timer_sclk == 4'd4 && counter_data != 8'd16) sclk <= ~sclk;
						else if(counter_data == 8'd16) sclk <= 1'd1;
					end
				else sclk <= 1'd1;
			end
	end

//sclk_buf
always @(posedge in_clk)
	begin
		if(in_reset == 1'd0)
			begin
				sclk_buf <= 1'd1;
			end
		else
			begin
				if(in_start == 1'h1) sclk_buf <= sclk;
				else if(in_start == 1'h0) sclk_buf <= 1'd1;
			end
	end


//counter_data
always @(posedge in_clk)
	begin
		if(in_reset == 1'h0)
			begin
				counter_data <= 8'd0;
			end
		else
			begin
				if(in_start == 1'h1 && sclk_pos == 1'd1 && counter_data < 8'd16) counter_data <= counter_data + 1;
				else if(in_start == 1'h0) counter_data <= 8'd0;
			end
	end	
	
	
	
//dout
always @(posedge in_clk)
	begin
		if(in_reset == 1'd0)
			begin
				dout <= 16'd0;
			end
		else
			begin
				if(in_start == 1'd1)
					begin
						if(counter_data == 8'd0) dout <= in_data_out;
						else if(counter_data > 8'd0 && counter_data < 8'd16 && sclk_neg == 1'd1) dout[15:0] <= {dout[14:0],1'd0};
					end
				else dout <= 16'd0;
			end
		end



//sdio
always @(posedge in_clk)
	begin
		if(in_reset == 1'd0)
			begin
				sdio <= 1'd0;
			end
		else
			begin
				if(in_start == 1'd1) sdio <= dout[15];
				else sdio <= 1'd0;
			end
	end

	
	
//timer_delay
always @(posedge in_clk)
	begin
		if(in_reset == 1'd0)
			begin
				timer_delay <= 4'd0;
			end
		else
			begin
				if(in_start == 1'd1)
					begin
						if(counter_data == 8'd16 && timer_delay < 4'd5) timer_delay <= timer_delay + 1;
						else if(counter_data != 8'd16) timer_delay <= 4'd0;
					end
				else timer_delay <= 4'd0;
			end
			
	end

	
//ss
always @(posedge in_clk)
	begin
		if(in_reset == 1'd0)
			begin
				ss <= 1'd1;
			end
		else
			begin
				if(busy == 1'd1) ss <= 1'd0;
				else ss <= 1'd1;
			end
		
	end
	
	
	
//busy 
always @(posedge in_clk)
	begin
		if(in_reset == 1'd0)
			begin
				busy <= 1'd0;
			end
		else
			begin
				if(in_start == 1'd1 && timer_delay != 4'd5) busy <= 1'd1;
				else if (timer_delay == 4'd5) busy <= 1'd0;
			end
	end

	



	
endmodule
